unit NDSPrint;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  RPCon, RPDefine, RPRave, StdCtrls, NDUtil, Registry, RVCsBars, RPBase,
  RPSystem, Buttons, ComCtrls;

type
  TMainForm = class(TForm)
    RaveProject: TRaveProject;
    SourceFileCXN: TRPCustomConnection;
    ReportSystem1: TReportSystem;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    GroupBox4: TGroupBox;
    SpeedButton1: TSpeedButton;
    FileNameEdit: TEdit;
    GroupBox3: TGroupBox;
    Label2: TLabel;
    Label3: TLabel;
    ReportLB: TListBox;
    DescMemo: TMemo;
    GroupBox1: TGroupBox;
    Button3: TButton;
    Button4: TButton;
    Button5: TButton;
    Button6: TButton;
    Button7: TButton;
    Button8: TButton;
    Button1: TButton;
    Button2: TButton;
    Button9: TButton;
    TabSheet3: TTabSheet;
    Memo1: TMemo;
    TabSheet4: TTabSheet;
    Memo2: TMemo;
    TabSheet5: TTabSheet;
    Memo3: TMemo;
    OpenDialog1: TOpenDialog;
    procedure SourceFileCXNOpen(Connection: TRPCustomConnection);
    procedure SourceFileCXNRestore(Connection: TRPCustomConnection);
    procedure SourceFileCXNFirst(Connection: TRPCustomConnection);
    procedure SourceFileCXNGetCols(Connection: TRPCustomConnection);
    procedure SourceFileCXNGetRow(Connection: TRPCustomConnection);
    procedure SourceFileCXNNext(Connection: TRPCustomConnection);
    procedure SourceFileCXNEOF(Connection: TRPCustomConnection;
      var Eof: Boolean);
    procedure Button1Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ReportLBClick(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure RaveProjectCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure RaveProjectDesignerSave(Sender: TObject);
    procedure Button9Click(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
  private
    { Private declarations }
    LineNum: longint;
    Line: string;
    FmtLine: string;
    NDParser: TNDParser;
    procedure ReadSourceLine;
    procedure LoadRegistry;
    procedure SaveRegistry;
    procedure SetParams;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses NDSPFont;

{$R *.DFM}

procedure TMainForm.SourceFileCXNOpen(Connection: TRPCustomConnection);
begin
  NDParser := TNDParser.Create;
end;

procedure TMainForm.SourceFileCXNRestore(Connection: TRPCustomConnection);
begin
  If Assigned(NDParser) then begin
    NDParser.Free;
    NDParser := nil;
  end; { if }
end;

procedure TMainForm.ReadSourceLine;

var
  EOLN: boolean;
  LastST: TSyntaxType;
  ThisST: TSyntaxType;
  RPTFStr: string;

begin { ReadSourceLine }
  Inc(LineNum);
  With NDParser do begin
    Line := '';
    FmtLine := '';
    LastST := stWS;
    Repeat
      GetToken;
      EOLN := TokenType in [ttEOLN,ttEOF];
      If not EOLN then begin
        Line := Line + TokenName;

      { Get syntax type of token }
        Case TokenType of
          ttOperator: begin
            If (TokenName = '{') or (TokenName = '}') or (TokenName = '//') or
             (TokenName = '(*') or (TokenName = '*)') then begin
              ThisST := stComment;
            end else begin
              ThisST := stOperator;
            end; { else }
          end;
          ttWS: ThisST := stWS;
          ttComment: ThisST := stComment;
          ttID: begin
            If Pos('|' + LowerCase(TokenName) + '|',ReservedWords) = 0 then begin
              ThisST := stID;
            end else begin
              ThisST := stReserved;
            end; { else }
          end;
          ttString: ThisST := stString;
          else ThisST := stNumber; { ttNumber }
        end; { case }

      { Build formatting string }
        RPTFStr := '';
        With SyntaxFonts[ThisST] do begin
          If SyntaxFonts[LastST].Bold <> Bold then begin
            If RPTFStr <> '' then begin
              RPTFStr := RPTFStr + RPTFInfix;
            end; { if }
            If Bold then begin
              RPTFStr := RPTFStr + RPTFBold + RPTFOn;
            end else begin
              RPTFStr := RPTFStr + RPTFBold + RPTFOff;
            end; { else }
          end; { if }
          If SyntaxFonts[LastST].Italic <> Italic then begin
            If RPTFStr <> '' then begin
              RPTFStr := RPTFStr + RPTFInfix;
            end; { if }
            If Italic then begin
              RPTFStr := RPTFStr + RPTFItalic + RPTFOn;
            end else begin
              RPTFStr := RPTFStr + RPTFItalic + RPTFOff;
            end; { else }
          end; { if }
          If SyntaxFonts[LastST].Underline <> Underline then begin
            If RPTFStr <> '' then begin
              RPTFStr := RPTFStr + RPTFInfix;
            end; { if }
            If Underline then begin
              RPTFStr := RPTFStr + RPTFUnderline + RPTFOn;
            end else begin
              RPTFStr := RPTFStr + RPTFUnderline + RPTFOff;
            end; { else }
          end; { if }
          If SyntaxFonts[LastST].Color <> Color then begin
            If RPTFStr <> '' then begin
              RPTFStr := RPTFStr + RPTFInfix;
            end; { if }
            RPTFStr := RPTFStr + RPTFColor + IntToStr(Color);
          end; { if }
        end; { with }

      { Add new token to FmtLine }
        If RPTFStr = '' then begin
          FmtLine := FmtLine + TokenName;
        end else begin
          FmtLine := FmtLine + RPTFPrefix + RPTFStr + RPTFSuffix + TokenName;
        end; { else }
        LastST := ThisST;
      end; { if }
    until EOLN;
  end; { with }
end;  { ReadSourceLine }

procedure TMainForm.SourceFileCXNFirst(Connection: TRPCustomConnection);
begin
  NDParser.LoadFromFile(FileNameEdit.Text);
  LineNum := 0;
  ReadSourceLine;
end;

procedure TMainForm.SourceFileCXNGetCols(Connection: TRPCustomConnection);
begin
  With Connection do begin
    WriteField('LineNum',dtInteger,6,'Line Number','');
    WriteField('Line',dtString,80,'','Plain Text Line');
    WriteField('FmtLine',dtString,80,'Formatted Text Line','');
    WriteField('CharCount',dtInteger,6,'Character Count for Line','');
  end; { with }
end;

procedure TMainForm.SourceFileCXNGetRow(Connection: TRPCustomConnection);

begin
  With Connection do begin
    WriteIntData('',LineNum);
    WriteStrData('',Line);
    WriteStrData('',FmtLine);
    WriteIntData('',Length(Line));
  end; { with }
end;

procedure TMainForm.SourceFileCXNNext(Connection: TRPCustomConnection);
begin
  ReadSourceLine;
end;

procedure TMainForm.SourceFileCXNEOF(Connection: TRPCustomConnection;
  var Eof: Boolean);
begin
  EOF := (NDParser.TokenType = ttEOF);
end;

procedure TMainForm.SetParams;

var
  FullFileName: string;
  FileName: string;
  FileExt: string;

begin { SetParams }
  FullFileName := ExpandFileName(FileNameEdit.Text);
  FileName := ExtractFileName(FullFileName);
  FileExt := ExtractFileExt(FullFileName);
  If FileExt <> '' then begin
    Delete(FileName,Length(FileName) - Length(FileExt) + 1,Length(FileExt));
    Delete(FileExt,1,1);
  end; { if }
  RaveProject.SetParam('FilePath',ExtractFilePath(FullFileName));
  RaveProject.SetParam('FileName',FileName);
  RaveProject.SetParam('FileExt',FileExt);
  RaveProject.SetParam('FullFileName',FullFileName);
  RaveProject.SetParam('FileDateTime',FloatToStr(FileDateToDateTime(FileAge(FullFileName))));
end;  { SetParams }

procedure TMainForm.Button1Click(Sender: TObject);

begin
  SetParams;
  RaveProject.Execute;
end;

procedure TMainForm.LoadRegistry;

var
  SyntaxType: TSyntaxType;

begin { LoadRegistry }
{ Read settings }
  With TRegistry.Create do try
    For SyntaxType := Low(TSyntaxType) to High(TSyntaxType) do begin
      With SyntaxFonts[SyntaxType] do begin
        OpenKey('\Software\Nevrona Designs\Source Printer\' + Name,true);
        If ValueExists('Bold') then begin
          Bold := ReadString('Bold') = '1';
        end; { if }
        If ValueExists('Italic') then begin
          Italic := ReadString('Italic') = '1';
        end; { if }
        If ValueExists('Underling') then begin
          Underline := ReadString('Underline') = '1';
        end; { if }
        If ValueExists('Color') then begin
          Color := TColor(StrToInt(ReadString('Color')));
        end; { if }
      end; { with }
    end; { for }
  finally
    Free;
  end; { with }
end;  { LoadRegistry }

procedure TMainForm.SaveRegistry;

var
  SyntaxType: TSyntaxType;

begin { SaveRegistry }
  With TRegistry.Create do try
    For SyntaxType := Low(TSyntaxType) to High(TSyntaxType) do begin
      With SyntaxFonts[SyntaxType] do begin
        OpenKey('\Software\Nevrona Designs\Source Printer\' + Name,true);
        WriteString('Bold',IntToStr(Ord(Bold)));
        WriteString('Italic',IntToStr(Ord(Italic)));
        WriteString('Underline',IntToStr(Ord(Underline)));
        WriteString('Color',IntToStr(Color));
      end; { with }
    end; { for }
  finally
    Free;
  end; { with }
end;  { SaveRegistry }

procedure TMainForm.Button3Click(Sender: TObject);

begin
  With TFontSettingsForm.Create(self) do try
    SyntaxType := TSyntaxType((Sender as TComponent).Tag);
    ShowModal;
    SaveRegistry;
  finally
    Free;
  end; { with }
end;

procedure TMainForm.FormCreate(Sender: TObject);

var
  SyntaxType: TSyntaxType;

begin
  RaveProject.Open;
  RaveProject.GetReportList(ReportLB.Items,true);
  For SyntaxType := Low(TSyntaxType) to High(TSyntaxType) do begin
    With SyntaxFonts[SyntaxType] do begin
      Case SyntaxType of
        stReserved: begin
          Bold := true;
          Italic := false;
          Underline := false;
          Color := clBlack;
          Name := 'Reserved Words';
        end;
        stID: begin
          Bold := false;
          Italic := false;
          Underline := false;
          Color := clBlack;
          Name := 'Identifiers';
        end;
        stNumber: begin
          Bold := false;
          Italic := false;
          Underline := false;
          Color := clGreen;
          Name := 'Numbers';
        end;
        stString: begin
          Bold := false;
          Italic := false;
          Underline := false;
          Color := clBlue;
          Name := 'Strings';
        end;
        stOperator: begin
          Bold := true;
          Italic := false;
          Underline := false;
          Color := clBlack;
          Name := 'Operators';
        end;
        stComment: begin
          Bold := false;
          Italic := true;
          Underline := false;
          Color := clRed;
          Name := 'Comments';
        end;
        stWS: begin
          Bold := false;
          Italic := false;
          Underline := false;
          Color := clBlack;
          Name := 'White Space';
        end;
      end; { case }
    end; { with }
  end; { for }
  LoadRegistry;
end;

procedure TMainForm.ReportLBClick(Sender: TObject);
begin
  RaveProject.SelectReport(ReportLB.Items[ReportLB.ItemIndex],true);
  RaveProject.ReportDescToMemo(DescMemo);
end;

procedure TMainForm.Button2Click(Sender: TObject);
begin
  SetParams;
  RaveProject.Design;
  RaveProject.SaveToFile(RaveProject.ProjectFile);
  RaveProject.GetReportList(ReportLB.Items,true);
end;

procedure TMainForm.RaveProjectCreate(Sender: TObject);
begin
  RVCsBars.RaveRegister;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  ReportLB.ItemIndex := 0;
  ReportLBClick(nil);
end;

procedure TMainForm.RaveProjectDesignerSave(Sender: TObject);
begin
  RaveProject.SaveToFile(RaveProject.ProjectFile);
end;

procedure TMainForm.Button9Click(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.SpeedButton1Click(Sender: TObject);
begin
  If OpenDialog1.Execute then begin
    FileNameEdit.Text := OpenDialog1.FileName;
  end; { if }
end;

end.
